% Initialize DJI M600 Pro hexacopter model with
% physical and control parameters
% Selectable configuration regarding mass and inertias
% NED ground coordinate system and aligned (X forward, Y right, Z down)
% body system are applied
% The user can give the run time, reference signals and the type of
% reference signals (as in DJI OSDK)
% Run simulation and plot results
%
% Institute for Computer Science and Control,  Budapest, Hungary
% Systems and Control Laboratory 2023.
% v3.0 12. June 2023.
%

clear all;
close all;

% Select mass and inertial setup:
% 1: NO payload, TB47S batteries
% 2: NO payload, TB48S batteries
INERTflag=2;

%% Create an integer flag based on DJI OSDK's flightCtrl function by commenting modes in/out
%For more ref see: https://developer.dji.com/onboard-api-reference/classDJI_1_1OSDK_1_1Control.html#aa89e5fd518382c94fb6149a5b9a6183b

%HorizontalCoordinate = dec2bin([0 0]);   % Ground/Earth (North East position / velocity reference)
HorizontalCoordinate = dec2bin([0 1]);    % Body (Tangential Normal position / velocity reference)

HorizontalLogic = dec2bin([0 0]);         % Pitch and roll angle [deg]
%HorizontalLogic = dec2bin([0 1]);         % Horizontal velocities [m/s]
%HorizontalLogic = dec2bin([1 0]);         % Horizontal positions [m]
%HorizontalLogic = dec2bin([1 1]);         % Pitch and roll angular rate NOT IMPLEMENTED

%VerticalLogic = dec2bin([0 0]);            % Velocity [m/s]
VerticalLogic = dec2bin([0 1]);            % Altitude [m]
%VerticalLogic = dec2bin([1 0]);            % Thrust [%] NOT IMPLEMENTED

YawLogic = dec2bin(0);                      % Angle [deg]
%YawLogic = dec2bin(1);                      % Angular rate [deg/s]

%StableMode = dec2bin(0);                   % Drone will not keep position
                                            %when velocity command is 0 NOT IMPLEMENTED
StableMode = dec2bin(1);                    % Drone will keep position when velocity command is 0

flightctrl_flag = bin2dec([HorizontalLogic; VerticalLogic; YawLogic; HorizontalCoordinate; StableMode]');

%% Define the four reference signals:
% Fill in empty flight measured references to be able to run simulation:
Xref=[0 0];
Yref=[0 0];
altref=[0 0];
yawref=[0 0];

% Give simulation runtime [s]
stop_time=60;

%% Reference type selectors for the different signals:
% type = 1 step
% type = 2 doublet
% type = 3 half doublet with period = full length
% type = 4 elongated doublet with half period zero value between plus and
% minus parts
Xtype=3;
Ytype=1;
Ztype=1;
YAWtype=1;

% X references
% Step reference
Xstep.amp=0;            % amplitude 
Xstep.zero=0;           % initial value (before step) 
Xstep.step=15;           % step time [s]

% Doublet reference
Xdoublet.amp=15;         % amplitude 
Xdoublet.zero=0;        % initial value (before doublet) 
Xdoublet.step=15;       % start time [s]
Xdoublet.period=15;     % period time [s]

% Y references
% Step reference
Ystep.amp=0;            % amplitude 
Ystep.zero=0;           % initial value (before step) 
Ystep.step=20;          % step time [s]

% Doublet reference
Ydoublet.amp=7;        % amplitude 
Ydoublet.zero=0;        % initial value (before doublet)
Ydoublet.step=15;       % start time [s]
Ydoublet.period=20;     % period time [s]

% Z references
% Step reference
Zstep.amp=0;            % amplitude 
Zstep.zero=70;          % initial value (before step) 
Zstep.step=16;          % step time [s]

% Doublet reference
Zdoublet.amp=3;         % amplitude 
Zdoublet.zero=0;        % initial value (before doublet)
Zdoublet.step=3;        % start time [s]
Zdoublet.period=10;     % period time [s]

% YAW references
% Step reference
YAWstep.amp=45/180*pi;  % amplitude 
YAWstep.zero=0/180*pi;  % initial value (before step) 
YAWstep.step=0;         % step time [s]

% Doublet reference
YAWdoublet.amp=45/180*pi;   % amplitude 
YAWdoublet.zero=0/180*pi;   % initial value (before doublet)
YAWdoublet.step=20;         % start time [s]
YAWdoublet.period=10;       % period time [s]

%% Other things:
% Run fix parameter initialization for DJI M600 model
Model_3D_init;

%% Give initial state values:
params.initial_speed=[0 0 0];           % NED velocity
params.initial_pos=[0 0 -70];           % NED position
params.initial_euler=[0 0 45/180*pi];   % Euler angles
params.initial_angvel=[0 0 0];          % angular rates

% Define wind strength 
% Overall resulting strength should be maximum 8m/s
params.WN=0;
params.WE=0;
params.WD=0; % should be zero!

%% Run Simulink:
sim('DJI_M600_Pro_model');

% Plot results
figure;
hold on;
plot(SvD.time,SvD.signals.values(:,1),'--','Linewidth',2);
plot(SvD.time,SvD.signals.values(:,2),'Linewidth',1.5);
grid
title('Vertical velocity tracking','FontSize', 12)
legend('SIM v_D ref','SIM v_D','Location','best','FontSize', 11);
xlabel('Time [s]','FontSize', 11);
ylabel('v_D [m/s]','FontSize', 11)
set(gca,'FontSize', 11);

figure;
hold on;
plot(h.time,h.signals.values(:,1),'--','Linewidth',2);
plot(h.time,h.signals.values(:,2),'Linewidth',1.5);
grid
title('Altitude tracking','FontSize', 12)
legend('SIM alt. ref','SIM alt.','Location','best','FontSize', 11);
xlabel('Time [s]','FontSize', 11);
ylabel('alt. [m]','FontSize', 11);
set(gca,'FontSize', 11);

figure;
hold on;
plot(SN.time,SN.signals.values(:,1),'--','Linewidth',2);
plot(SN.time,SN.signals.values(:,2),'Linewidth',1.5);
grid
title('North position tracking','FontSize', 12)
legend('SIM Pos. ref.','SIM Pos.','Location','best','FontSize', 11);
xlabel('Time [s]','FontSize', 11);
ylabel('North [m]','FontSize', 11);
set(gca,'FontSize', 11);

figure;
hold on;
plot(SE.time,SE.signals.values(:,1),'--','Linewidth',2);
plot(SE.time,SE.signals.values(:,2),'Linewidth',1.5);
grid
title('East position','FontSize', 12)
legend('SIM Pos. ref.','SIM Pos.','Location','best','FontSize', 11);
xlabel('Time [s]','FontSize', 11);
ylabel('East [m]','FontSize', 11);
set(gca,'FontSize', 11);

figure;
hold on;
plot(SE.signals.values(:,2),SN.signals.values(:,2));
grid
title('North-East trajectory','FontSize', 12)
legend('SIM','Location','best','FontSize', 11);
xlabel('East [m]','FontSize', 11);
ylabel('North [m]','FontSize', 11);
set(gca,'FontSize', 11);

figure;
hold on;
plot(SvT.time,SvT.signals.values(:,1),'--','Linewidth',2);
plot(SvT.time,SvT.signals.values(:,2),'Linewidth',1.5);
grid
title('Tangential velocity tracking','FontSize', 12)
legend('SIM v_T ref','SIM v_T','Location','best','FontSize', 11);
xlabel('Time [s]','FontSize', 11);
ylabel('v_T [m/s]','FontSize', 11);
set(gca,'FontSize', 11);

figure;
hold on;
plot(SvN.time,SvN.signals.values(:,1),'--','Linewidth',2);
plot(SvN.time,SvN.signals.values(:,2),'Linewidth',1.5);
grid
title('Normal velocity tracking','FontSize', 12)
legend('SIM v_N ref','SIM v_N','Location','best','FontSize', 11);
xlabel('Time [s]','FontSize', 11);
ylabel('v_N [m/s]','FontSize', 11)

figure;
hold on;
plot(SvNo.time,SvNo.signals.values(:,1),'--','Linewidth',2);
plot(SvNo.time,SvNo.signals.values(:,2),'Linewidth',1.5);
grid
title('North velocity tracking','FontSize', 12)
legend('SIM v_N ref','SIM v_N','Location','best','FontSize', 11);
xlabel('Time [s]','FontSize', 11);
ylabel('v_N [m/s]','FontSize', 11);
set(gca,'FontSize', 11);

figure;
hold on;
plot(SvEa.time,SvEa.signals.values(:,1),'--','Linewidth',2);
plot(SvEa.time,SvEa.signals.values(:,2),'Linewidth',1.5);
grid
title('East velocity tracking','FontSize', 12)
legend('SIM v_E ref','SIM v_E','Location','best','FontSize', 11);
xlabel('Time [s]','FontSize', 11);
ylabel('v_E [m/s]','FontSize', 11);
set(gca,'FontSize', 11);

figure;
hold on;
plot(phi.time,phi.signals.values(:,1),'--','Linewidth',2);
plot(phi.time,phi.signals.values(:,2),'Linewidth',1.5);
grid
title('Roll angle tracking','FontSize', 12)
legend('SIM \phi ref','SIM \phi','Location','best','FontSize', 11);
xlabel('Time [s]','FontSize', 11);
ylabel('\phi [deg]','FontSize', 11);
set(gca,'FontSize', 11);

figure;
hold on;
plot(theta.time,theta.signals.values(:,1),'--','Linewidth',2);
plot(theta.time,theta.signals.values(:,2),'Linewidth',1.5);
grid
title('Pitch angle tracking','FontSize', 12)
legend('SIM \theta ref','SIM \theta','Location','best','FontSize', 11);
xlabel('Time [s]','FontSize', 11);
ylabel('\theta [deg]','FontSize', 11);
set(gca,'FontSize', 11);

figure;
hold on;
plot(psi.time,psi.signals.values(:,1),'--','Linewidth',2);
plot(psi.time,psi.signals.values(:,2),'Linewidth',1.5);
grid
title('Yaw angle tracking','FontSize', 12)
legend('SIM \psi ref','SIM \psi','Location','best','FontSize', 11);
xlabel('Time [s]','FontSize', 11);
ylabel('\psi [deg]','FontSize', 11);
set(gca,'FontSize', 11);